@@ -10,7 +10,7 @@ use Data::Dumper;
 ## no critic (DotMatch,LineBoundary,Sigils,Punctuation,Quotes,Magic,Checked)
 ## no critic (NamingConventions::Capitalization,BracedFileHandle)
 
-our $VERSION = '1.12_01';
+our $VERSION = '1.13_01';
 our $DEBUG = $ENV{DEBUG} || 0;
 our $XS_LOADED = 0;
 eval { bootstrap Sys::CpuAffinity $VERSION; $XS_LOADED = 1 };
@@ -54,10 +54,12 @@ sub getAffinity {
     my ($pid, %flags) = @_; # %flags reserved for future use
     my $wpid = $pid;
 
+    # 2020-12-18: taskset only returns 32 bits? Give xs_sched more priority
+    
     my $mask = 0
-	|| _getAffinity_with_taskset($pid)
 	|| _getAffinity_with_xs_sched_getaffinity($pid)
 	|| _getAffinity_with_xs_pthread_self_getaffinity($pid)
+	|| _getAffinity_with_taskset($pid)
 	|| _getAffinity_with_BSD_Process_Affinity($pid)
 	|| _getAffinity_with_xs_freebsd_getaffinity($pid)
 	|| _getAffinity_with_cpuset($pid)
@@ -128,8 +130,8 @@ sub setAffinity {
     return _setAffinity_with_Win32API($pid,$mask)
 	|| _setAffinity_with_xs_win32($pid,$mask)
 	|| _setAffinity_with_Win32Process($pid,$mask)
-	|| _setAffinity_with_taskset($pid,$mask)
 	|| _setAffinity_with_xs_sched_setaffinity($pid,$mask)
+	|| _setAffinity_with_taskset($pid,$mask)
 	|| _setAffinity_with_BSD_Process_Affinity($pid,$mask)
 	|| _setAffinity_with_xs_freebsd_setaffinity($pid,$mask)
         || _setAffinity_with_xs_processor_affinity($pid,$mask)
@@ -1030,7 +1032,24 @@ sub _getAffinity_with_xs_freebsd_getaffinity {
     my $pid = shift;
     return 0 if !defined &xs_getaffinity_freebsd;
     my @mask = ();
-    my $ret = xs_getaffinity_freebsd($pid,\@mask);
+    my $ret = xs_getaffinity_freebsd($pid,\@mask,0);
+    if ($ret == 0) {
+	return 0;
+    }
+    return _arrayToMask(@mask);
+}
+
+sub _getAffinity_with_xs_freebsd_getaffinity_debug {
+    my $pid = shift;
+    if (!defined &xs_getaffinity_freebsd) {
+        if ($^O =~ /bsd/) {
+            warn "\$^O=$^O, xs_getaffinity_freebsd not defined";
+        }
+        return;
+    }
+    my @mask = ();
+    my $ret = xs_getaffinity_freebsd($pid,\@mask,1);
+    warn "return value from xs_getaffinity_freebsd: $ret";
     if ($ret == 0) {
 	return 0;
     }
@@ -1238,10 +1257,10 @@ sub _setAffinity_with_taskset {
 }
 
 sub _setAffinity_with_xs_sched_setaffinity {
-    my ($pid,$mask) = @_;
+    my ($pid,$mask,$debug) = @_;
     return 0 if !defined &xs_sched_setaffinity_set_affinity;
     my @mask = _maskToArray($mask);
-    return xs_sched_setaffinity_set_affinity($pid,\@mask);
+    return xs_sched_setaffinity_set_affinity($pid,\@mask,0+!!$debug);
 }
 
 sub _setAffinity_with_BSD_Process_Affinity {
@@ -1638,7 +1657,7 @@ Sys::CpuAffinity - Set CPU affinity for processes
 
 =head1 VERSION
 
-Version 1.12_01
+Version 1.13_01
 
 =head1 SYNOPSIS
 
@@ -1880,7 +1899,7 @@ Marty O'Brien, C<< <mob at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-2016 Marty O'Brien.
+Copyright 2010-2017 Marty O'Brien.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
